/************************************************************************************************************************************************

File: 		WinderA1
Author:		Gordon D. Dick, P. Eng. (retired)
Description: This is Winder11 modified to work on the ww board for the article. Some of the IO is different now. Further, the lcd is now a 16x2
whereas previously it was 40x2.
		Last work on this code was done on Feb 20, 2015.

*************************************************************************************************************************************************/



/*					This code was developed using MPLAB v8.83 and the HITECH C v9.65 free compiler for use with a PIC16F876.					    */



/************************************************************************************************************************************************
																	Revision History
*************************************************************************************************************************************************
		Nov 2014: Initial code for project done on a PIC18F4520 board.		Rev 1.0
		Jan 2015: Modified code to run on EST PIC16F876 board.				Rev 1.1
		Feb 2015: Modified code to run on a WW PIC16F876 board.				Rev 1.2

************************************************************************************************************************************************/


/************************************************************************************************************************************************
																	Includes
************************************************************************************************************************************************/
#include	<htc.h>						//generic include for all devices
#include	<stdlib.h>
#include	"LCD.h"


//NOTE:****************************
//Use the Configure tab to: HS oscillator, WDT disabled,  BOR enabled, RB3 is digital IO, code protection off, unprotected program memory may be
//written to by EECON. Configuration word is 0x3F7A.
//*********************************


/*************************************************************************************************************************************************
																	Defines
*************************************************************************************************************************************************/
#define FOSC 4000000
// Port I/O directions (0 = output, 1 = input)
#define PORTA_DIR	0b11110000				//RA0:RA3 need to be outputs for keypad, RA4 and RA5 are not used
#define PORTB_DIR	0b0001001				//RB7:RB4 are LCD data lines, RB2 is CS, RB1 is RS, RB0 & RB3 not used
#define PORTC_DIR	0b11110000				//RC4:RC7 are keypad inputs, RC0:RC3 are CCP outputs and direction control for Easy Steppers
//Steps for the Spool designated as Y axis are produced by CCP1 ISR: step signal is on RC2.
//Steps for the Leveler designated as X axis are produced by CCP2 ISR: step signal is on RC1.
#define COL1 RC4
#define COL2 RC5
#define COL3 RC6
#define COL4 RC7
#define ROW1 RA0
#define ROW2 RA1
#define ROW3 RA2
#define ROW4 RA3

/*************************************************************************************************************************************************
													I/O Definitions( for keypad on WW Board)
*************************************************************************************************************************************************/
/*					Need 8 IO for keypad:			RA0 : RA3	(Rows), RC4:RC7	(Cols)
					Need 2 outputs for steps:		RC1(CCP2 ): RC2(CCP1)
					Need 2 outputs for direction:	RC0 & RC3
					Need 4 outputs for LCD data:	RB4 : RB7
					Need 2 outputs for LCD cntrl:	RB1 : RB2

					Unused I/O:	RB0 & RB3, RA4 & RA5

		Y axis (winder) step pulse are produced by (CCP1)RC2 and the direction pin is RC3
		X axis (leveler) step pulses are produced by (CCP2)RC1 and the direction pin is RC0
*/


/*************************************************************************************************************************************************
													Keypad Marking Useage
*************************************************************************************************************************************************/
/*					*		X	7 for CCW
					#		Y	9 for CCW
					C		not used yet
					D		Go or Do It
					A		Alter
					B		not used yet


*/

/*************************************************************************************************************************************************
															GLOBAL VARIABLES
*************************************************************************************************************************************************/
	char ucState = 0;
	char ucTicCount = 0;
	char ucTicFlag = 0;											//indicates a new tick when set
	char ucPass = 0;											//pass counter for blinking colon
	unsigned int uiComp1;										// 0x0271 for 100Hz
	unsigned int uiComp2;										// 0x058A for 44Hz
	unsigned int uiCp1;
	unsigned int uiCp2;
    unsigned int uiTurnsMax;
    unsigned int uiLayerLength;
	unsigned char ucStepY = 0;
	unsigned int uiTurns = 0;
	unsigned int uiStepX = 0;
	unsigned int uiLayers = 0;
	unsigned char ucNewSec = 0;
    unsigned int uiElapsedSecs = 0;
    unsigned int uiTemp = 0;
	unsigned char ucDone = 0;



/*************************************************************************************************************************************************
															GLOBAL CONSTANTS
*************************************************************************************************************************************************/
	enum State { Waiting, Winding, Jogging1, Jogging2, Pause, Done };
	const unsigned int uiWeightTable[] = { 10000, 1000, 100, 10, 1 };
	const unsigned int uiAsciiTime[] = { 0x3030,0x3031,0x3032,0x3033,0x3034,0x3035,0x3036,0x3037,0x3038,0x3039,0x3130,0x3131,0x3132,0x3133,0x3134,
										 0x3135,0x3136,0x3137,0x3138,0x3139,0x3230,0x3231,0x3232,0x3233,0x3234,0x3235,0x3236,0x3237,0x3238,0x3239,
										 0x3330,0x3331,0x3332,0x3333,0x3334,0x3335,0x3336,0x3337,0x3338,0x3339,0x3430,0x3431,0x3432,0x3433,0x3434,
										 0x3435,0x3436,0x3437,0x3438,0x3439,0x3530,0x3531,0x3532,0x3533,0x3534,0x3535,0x3536,0x3537,0x3538,0x3539 };
	const char cFirstLine[] =	"Winder by GD, EE";			//16 char strings
	const char cSecondLine[] =	"Rev 1.2 Feb 2015";			//16 char strings
	const char cBlankLine[] =	"                ";
	const char cMess1[]		= 	"Press A to alter";
	const char cMess2[]		= 	"any other for No";
	const char cCoilTurnsBlank[] 		=	"    ";
	const char cCoilStepsBlank[] 		=	"     ";
	const char cLevelerLayersBlank[] 	=	"    ";
	const char cLevelerStepsBlank[]		=	"     ";
	const char cTimeBlank[]				= 	"        ";
	const char cTimeColons[]			=	"  :  ";
	const char cBlankTurns[] 	= { "    "};
	const char cBlankLayers[] 	= { "    " };
	const char cBlankStepY[] 	= { "     " };
	const char cBlankStepX[] 	= { "     " };
	const char cWaitingMess[] 	=	"Wait";
	const char cWindingMess[] 	=	"Wind";
	const char cJogging1Mess[]	=	"Jog1";
	const char cJogging2Mess[]	=	"Jog2";
	const char cPauseMess[] 	=	"Paus";
	const char cDoneMess[]		=	"Done";
	const char cWinder[]		= "Winder rate?    ";
	const char cLeveler[]		= "Leveler rate?   ";
	const char cTurns[]			= "Number turns?   ";
	const char cLength[]		= "Layer Length?   ";
	const char cKeyArray[17] = {' ','1','2','3','A','4','5','6','B','7','8','9','C','*','0','#','D'};


/*************************************************************************************************************************************************
																Prototypes
*************************************************************************************************************************************************/
	void Timer0_ISR( void );
	void GoJog1( unsigned char cKey );
	void WindingPause( char cKey, char * cPtrStepXString, char * cPtrStepYString );
	void Finished( void );
	void Jog1X( void );
	void Jog1Y( void );
	void DoneJog1( char cKey );
	void Jog2X( char * cPtrLayersString, char * cPtrStepXString  );
	void Jog2Y( char * cPtrTurnsString, char * cPtrStepYString );
	void Jog2Pause( char cKey );
	void PauseJog2( char cKey );
	void PauseWinding( char cKey );
	void DoneWait( char cKey, unsigned char * ucPtrTime );
	void ShowElapsedTime( const unsigned int * uiPtr, unsigned char * ucPtr );
	void CalculateElapsedTime( unsigned char * ucPtr );
	void ShowParameters( void );
    char KeyPadConvert(char key);
    char KeyPadReadColumn(void);
    char KeyPadScan(void);
	char GetKey( void );
	void Change( void );
	unsigned int GetNum( void );



/*************************************************************************************************************************************************
																	Main
*************************************************************************************************************************************************/
void main(void)
{
	char cTemp;
	char ucPassOld = 0;
	char cKey = 0;

	unsigned char ucTime[ 3 ] = { 0, 0, 0 };						//Hours, Mins, Secs
	char cTurnsString[6] = { 0 };
	char cStepYString[4] = { 0 };
	char cLayersString[6] = { 0 };
	char cStepXString[4] = { 0 };


	TRISC = PORTC_DIR;
	TRISB = PORTB_DIR;
	TRISA = PORTA_DIR;

	ADCON1 = 0x06;													//want Port A all digital

	OPTION_REG = 0x05;												//TMR0 uses internal clock, /64, this gives a 16.384ms tick interval
	INTCON |= 0x20;													//enable TMR0 ints

	T1CON = 0x31;													//divide by 8,clock Fosc/4,Timer1 ON

	__delay_ms( 1000 );												//for the lcd
	LCDInit();
	LCDConstString2Spot( 1, 0, cFirstLine );						//cFirstLine[] = "Winder by GD, EE"
	LCDConstString2Spot( 2, 0, cSecondLine );						//cSecondLine[]= "Rev 1.2 Feb 2015"
	__delay_ms( 3000 );
	LCDConstString2Spot( 1, 0, cBlankLine );	

//Show messages preceeding winding parameters.
	LCDConstString2Spot( 1, 0, cMess1 );							//cMess1[] = "Press A to alter"
	LCDConstString2Spot( 2, 0, cMess2 );							//cMess2[] = "any other for No"
	__delay_ms( 5000 );												//wait a little

	LCDConstString2Spot( 1, 0, cBlankLine );						//erase
	LCDConstString2Spot( 2, 0, cBlankLine );						//		both lines
	ShowParameters();												//Winding parameters are now showing

//Pressing 'A' here will call Change(), any other key will go to Waiting State.
	do
	{
		cTemp = GetKey();
	}while( cTemp == ' ' );											//wait for any button
	
		if( cTemp == 'A' )											//If 'A' was pressed,
		{
			LCDConstString2Spot( 1, 0, cBlankLine );				//erase
			LCDConstString2Spot( 2, 0, cBlankLine );				//		both lines
			Change();												//					call Change()
			ShowParameters();										//Show new parameters after a change
			cTemp = 0;
		}

//Modified parameters are showing now. Pressing any key will proceed.
	do
	{
		cTemp = GetKey();
	}while( cTemp == ' ' );											//wait for any button

	LCDConstString2Spot( 1, 0, cBlankLine );						//erase
	LCDConstString2Spot( 2, 0, cBlankLine );						//		both lines

//CCP1CON and CCP2CON are initialized elsewhere. And so are CCPR1H, CCPR1L, CCPR2H, and CCPR2L.

	LCDConstString2Spot( 2, 1, cWaitingMess );

//Establish levels for direction control signals.
	PORTC |= 0x01;													//Winder direction signal is always high to wind
	PORTC &= ~0x08;													//Leveler should always start out low

	INTCON |= 0x80;													//enable global ints

	while(1)
	{
		while( ucTicFlag == 0 );									//hang the processor here until a new tick arrives
		ucTicFlag = 0;												//clear the flag for next time

//It isn't necessary for the code below to be executed at 'processor speed'. Once every tick is adequate.

		switch( ucState )//This is the State Machine that controls the Winder.
		{
			case Waiting:											//check for a D, *, #  button press and show the "Waiting" message
				PORTC |= 0x01;										//Winder direction signal is always high to wind
				PORTC &= ~0x08;										//Leveler should always start out low
				LCDConstString2Spot( 1, 7, cBlankTurns );
				LCDConstString2Spot( 2, 7, cBlankLayers );
				GoJog1( GetKey() );									//from here can go to Jog1 or Winding
				break;

			case Winding:											//check for any button press and show the "Winding" message
				if( ucNewSec )
				{
					LCDConstString2Spot( 1, 0, cTimeColons );		//put the colons up for elapsed time
					ucNewSec = 0;
					CalculateElapsedTime( ucTime );
					ShowElapsedTime( uiAsciiTime, ucTime );
				}
				Finished();
				xtoa( uiTurns, uiWeightTable, cTurnsString );
				LCDString2Spot( 1, 7, cTurnsString );
				xtoa( uiLayers, uiWeightTable, cLayersString );
				LCDString2Spot( 2, 7, cLayersString );
				WindingPause( GetKey(), cStepXString, cStepYString );
				break;

			case Jogging1:											//jog only while X or Y is pressed and show the "Jog1" message
				Jog1X();
				Jog1Y();
				DoneJog1( GetKey() );								//just watch for 'D' button
				break;

			case Jogging2:											//jog only while X or Y is pressed and show the "Jog2" message
				Jog2X( cLayersString, cStepXString  );
				Jog2Y( cTurnsString, cStepYString );
				Jog2Pause( GetKey() );								//just watch for 'D' button
				break;

			case Pause:												//check for G, X, or Y button press and show the "Pause" message
				cKey = GetKey();
				PauseJog2( cKey );
				PauseWinding( cKey );
				break;

			case Done:
				LCDConstString2Spot( 2, 1, cDoneMess );
				DoneWait( GetKey(), ucTime );
				
		}//End of Winder State Machine switch.


//This code executes once per second.

		if( ucPass != ucPassOld )
		{
			ucPassOld = ucPass;										//update the pass count
			Blink( ucPass );										//					and blink the colon
			ucNewSec++;												//signal a new second
		}//end of 'once per sec' if


	}//end of continuous loop
}//end of main




/*************************************************************************************************************************************************
													Functions
*************************************************************************************************************************************************/


/************************************************************************************************************************************
		GetNum()gets an ascii string representing a decimal number and converts it to an int
************************************************************************************************************************************/
unsigned int GetNum( void )
{		
	unsigned char ucDone = 0;
	unsigned char ucCount = 0;
	char cKey;
	unsigned long int ulEntry;
	char NumString[6];

//Make sure that the array we are going to use is empty.
	for( ucCount = 0; ucCount < 6; ucCount++)
		NumString[ ucCount ] = 0;

	ucCount = 0;
	do{
		do{														
			cKey = GetKey();
		}while( cKey == ' ' );									//read the keypad until we get a press
			if( cKey == 'D' )									//Was it a 'D'? If Yes,
			ucDone = 1;											//					signal Done entering numbers.
		if ( ( cKey >= '0' ) && ( cKey  <= '9' ) )				//Do we have valid number? If Yes,
		{														//						....
			CursorXY( ucCount, 2 );								//							...
			LCDPutc( cKey );									//									show it.
			NumString[ ucCount ] = cKey;						//put the ascii code into the buffer
			ucCount++;											//count a valid number
			if( ucCount > 4 )									//Only want 5 digits at most. Do we have 5?
				ucDone = 1;										//If Yes, signal done.
		}
		__delay_ms( 20 );
	}while( !ucDone  );
	ulEntry = strtoul( NumString, 0, 0 );						//convert the ascii string to a number
	return (unsigned int) ulEntry;
}


/************************************************************************************************************************************
		Change(): 	Allows the winding parameters to be changed.
************************************************************************************************************************************/
void Change( void )
{
	unsigned char ucTemp;
	unsigned int uiTemp;


//Do we want to change the Winder stepping rate? The data in uiComp1 establishes the Winder stepping rate: use Byte0(LSByte) and Byte1(MSByte).
	LCDConstString2Spot( 1, 0, cWinder );						//cWinder[]	= "Winder rate?    "
	do{															//the answer must be A to change or any other key for No
		ucTemp = GetKey();
	}while( ucTemp == ' ' );									//wait for any button
	if( ucTemp == 'A' )											//If the answer is Yes, change eeprom data for winder stepping rate.
	{
		LCD_RS = 0;												// write commands
		LCDWrite( 0x0F ); 										// Display On, Cursor ON, Blink
		CursorXY( 0, 2 );
		uiTemp = GetNum();										//number obtained is returned
		ucTemp = uiTemp & 0x00FF;
		eeprom_write( 0, ucTemp );								// Write LSByte to address 0 location
		ucTemp = uiTemp >> 8;
		eeprom_write( 1, ucTemp );								// Write MSByte to address 1 location
	}
		LCDConstString2Spot( 1, 0, cBlankLine );				//erase
		LCDConstString2Spot( 2, 0, cBlankLine );				//		both lines
		LCD_RS = 0;												// write commands
		LCDWrite( 0x0C ); 										// Display On, Cursor OFF, No Cursor Blink

//Do we want to change the Leveler stepping rate? The data in uiComp2 establishes the Leveler stepping rate: use Byte2(LSByte) and Byte3(MSByte).
	LCDConstString2Spot( 1, 0, cLeveler );						//cLeveler[] = "Leveler rate?   "
	do{															//the answer must be A to change or any other key for No
		ucTemp = GetKey();
	}while( ucTemp == ' ' );									//wait for any button
	if( ucTemp == 'A' )											//If the answer is Yes, change eeprom data for winder stepping rate.
	{
		LCD_RS = 0;												// write commands
		LCDWrite( 0x0F ); 										// Display On, Cursor ON, Blink
		CursorXY( 0, 2 );
		uiTemp = GetNum();										//number obtained is returned
		ucTemp = uiTemp & 0x00FF;
		eeprom_write( 2, ucTemp );								// Write LSByte to address 2 location
		ucTemp = uiTemp >> 8;
		eeprom_write( 3, ucTemp );								// Write MSByte to address 3 location
	}
		LCDConstString2Spot( 1, 0, cBlankLine );				//erase
		LCDConstString2Spot( 2, 0, cBlankLine );				//		both lines
		LCD_RS = 0;												// write commands
		LCDWrite( 0x0C ); 										// Display On, Cursor OFF, No Cursor Blink

//Do we want to change the Layer Length? The data in uiLayerLength establishes the number of Leveler steps per layer: use Byte4(LSByte) and Byte5(MSByte).
	LCDConstString2Spot( 1, 0, cLength );						//cLength[]	= "Layer Length?   "
	do{															//the answer must be A to change or any other key for No
		ucTemp = GetKey();
	}while( ucTemp == ' ' );									//wait for any button
	if( ucTemp == 'A' )											//If the answer is Yes, change eeprom data for winder stepping rate.
	{
		LCD_RS = 0;												// write commands
		LCDWrite( 0x0F ); 										// Display On, Cursor ON, Blink
		CursorXY( 0, 2 );
		uiTemp = GetNum();										//number obtained is returned
		ucTemp = uiTemp & 0x00FF;
		eeprom_write( 4, ucTemp );								// Write LSByte to address 4 location
		ucTemp = uiTemp >> 8;
		eeprom_write( 5, ucTemp );								// Write MSByte to address 5 location
	}
		LCDConstString2Spot( 1, 0, cBlankLine );				//erase
		LCDConstString2Spot( 2, 0, cBlankLine );				//		both lines
		LCD_RS = 0;												// write commands
		LCDWrite( 0x0C ); 										// Display On, Cursor OFF, No Cursor Blink

//Do we want to change the Turns? The data in uiTurnsMax establishes the total number of coil turns: use Byte6(LSByte) and Byte7(MSByte).
	LCDConstString2Spot( 1, 0, cTurns );						//cTurns[] = "Number turns?   "
	do{															//the answer must be A to change or any other key for No
		ucTemp = GetKey();
	}while( ucTemp == ' ' );									//wait for any button
	if( ucTemp == 'A' )											//If the answer is Yes, change eeprom data for winder stepping rate.
	{
		LCD_RS = 0;												// write commands
		LCDWrite( 0x0F ); 										// Display On, Cursor ON, Blink
		CursorXY( 0, 2 );
		uiTemp = GetNum();										//number obtained is returned
		ucTemp = uiTemp & 0x00FF;
		eeprom_write( 6, ucTemp );								// Write LSByte to address 6 location
		ucTemp = uiTemp >> 8;
		eeprom_write( 7, ucTemp );								// Write MSByte to address 7 location
	}
		LCDConstString2Spot( 1, 0, cBlankLine );				//erase
		LCDConstString2Spot( 2, 0, cBlankLine );				//		both lines
		LCD_RS = 0;												// write commands
		LCDWrite( 0x0C ); 										// Display On, Cursor OFF, No Cursor Blink
}


/*************************************************************************************************************************************************
	GetKey() reads the button port and returns the code for a valid keypress/release, or 0 if no key pressed.
*************************************************************************************************************************************************/
char GetKey( void )
{
	char cTemp, cPress;

	cPress = KeyPadScan(); 										//check for a button, code returned, ' ' for no key
		if( cPress != ' ' )
		{	
			cTemp = cPress;										//might be a keypress, keep a copy
			__delay_ms( 20 );									//wait for some bounce
			cPress = KeyPadScan();								//read the keypad again
			if( cPress == cTemp )								//Same key read?
				while( ' ' != KeyPadScan() );					//If Yes, scan keypad until button release.
			else
				return 0;										//If No, bad press, return 0.
		}
	return cPress;
}


/************************************************************************************************************************************
	KeyPadConvert() fetches ascii codes based on keypresses.
*************************************************************************************************************************************/
char KeyPadConvert( char cKey )
{
    cKey = cKeyArray[ cKey ];
    return( cKey );
}


/************************************************************************************************************************************
	KeyPadReadColumn() checks each column for a press.
*************************************************************************************************************************************/
char KeyPadReadColumn( void )
{
    if( COL1 ) {  		// Column1 is 1
        return (1);
    }
    else
		if( COL2 ) {  	// Column2 is 1
        	return (2);
    	}
    	else
			if( COL3 ) {  	// Column3 is 1
        		return (3);
    		}
    		else
				if( COL4 ) {  	// Column4 is 1
        			return (4);
    			}
    			else {  //No column is 1
        			return(0);
    			}
}


/************************************************************************************************************************************
	KeyPadScan() checks each column and row for a keypress. If no key is pressed, 0 is returned.
*************************************************************************************************************************************/
char KeyPadScan( void )
{
    char key = 0;

    ROW1 = 1;													//activate row 1
    key = KeyPadReadColumn();
    if (key == 0)												//No key in row 1
	{
        ROW1 = 0;
        ROW2 = 1;												//activate row 2
        key = KeyPadReadColumn() + 4;
        if (key == 4)											//No key in row 2
		{
            ROW2 = 0;
	        ROW3 = 1;											//activate row 3
            key = KeyPadReadColumn() + 8;
            if (key == 8)										//No key in row 3
			{
                ROW3 = 0;
                ROW4 = 1;										//activate row 4
                key = KeyPadReadColumn() + 12;
                if (key == 12)									//No key in row 4
				{
                    key = 0;
                }
            }
        }
    }
    PORTA = 0;
    key = KeyPadConvert( key );
    return( key );
}


/*************************************************************************************************************************************************
		ShowParameters()
*************************************************************************************************************************************************/
void ShowParameters( void )
{
	unsigned char cTempBuff[6];

//Show the initial values for the winding parameters.

//	The data in uiCp1 establishes the Winder stepping rate: use Byte0(LSByte) and Byte1(MSByte).
	uiCp1 = eeprom_read( 1 );
	uiCp1 <<= 8;
	uiCp1 |= eeprom_read( 0 );
	xtoa( uiCp1, uiWeightTable, cTempBuff );
	CursorXY( 0, 1 );
	LCDPutc( 'W' ); LCDPutc( ':' );
	LCDString2Spot( 1, 2, cTempBuff );
//	The data in uiCp2 establishes the Leveler stepping rate: use Byte2(LSByte) and Byte3(MSByte).
	uiCp2 = eeprom_read( 3 );
	uiCp2 <<= 8;
	uiCp2 |= eeprom_read( 2 );
	xtoa( uiCp2, uiWeightTable, cTempBuff );
	CursorXY( 8, 1 );
	LCDPutc( 'L' ); LCDPutc( ':' );
	LCDString2Spot( 1, 10, cTempBuff );
//	The data in uiLayerLength establishes the number of Leveler steps per layer: use Byte4(LSByte) and Byte5(MSByte).
	uiLayerLength = eeprom_read( 5 );
	uiLayerLength <<= 8;
	uiLayerLength |= eeprom_read( 4 );
	xtoa( uiLayerLength, uiWeightTable, cTempBuff );
	CursorXY( 8, 2 );
	LCDPutc( 'L' ); LCDPutc( ':' );
	LCDString2Spot( 2, 10, cTempBuff );
//	The data in uiTurnsMax establishes the total number of coil turns: use Byte6(LSByte) and Byte7(MSByte).
	uiTurnsMax = eeprom_read( 7 );
	uiTurnsMax <<= 8;
	uiTurnsMax |= eeprom_read( 6 );
	xtoa( uiTurnsMax, uiWeightTable, cTempBuff );
	CursorXY( 0, 2 );
	LCDPutc( 'T' ); LCDPutc( ':' );
	LCDString2Spot( 2, 2, cTempBuff );
}


/*************************************************************************************************************************************************
	CalculateElapsedTime()
*************************************************************************************************************************************************/
void CalculateElapsedTime( unsigned char * ucPtr )
{
	*( ucPtr + 2 ) += 1;
	if( *( ucPtr + 2 ) > 59 )
		*( ucPtr + 2 ) = 0, *( ucPtr + 1 ) += 1;
	if( *( ucPtr + 1 ) > 59 )
		*( ucPtr + 1 ) = 0, *ucPtr += 1;
}


/*************************************************************************************************************************************************
	ShowElapsedTime()
*************************************************************************************************************************************************/
void ShowElapsedTime( const unsigned int * uiPtr, unsigned char * ucPtr )
{
	unsigned char ucTemp;
//Not going to show hours now.
//	ucTemp =  ( *( uiPtr + * ucPtr ) & 0xFF00 ) >> 8;
//	CursorXY( 0, 1);
//	LCDPutc( ucTemp );
//	LCDPutc( *( uiPtr + *ucPtr ) );
	ucTemp = ( *( uiPtr + *( ucPtr + 1 ) ) & 0xFF00 ) >> 8;
	CursorXY( 0, 1);
	LCDPutc( ucTemp );
	LCDPutc( *( uiPtr + *( ucPtr + 1 ) ) );
	ucTemp = ( *( uiPtr + *( ucPtr + 2 ) ) & 0xFF00 ) >> 8;
	CursorXY( 3, 1);
	LCDPutc( ucTemp );
	LCDPutc( *( uiPtr + *( ucPtr + 2 ) ) );
}


/*************************************************************************************************************************************************
	DoneWait() checks to see if we want to wind another coil.
*************************************************************************************************************************************************/
void DoneWait( char cKey, unsigned char * ucPtrTime )
{
	if( cKey == 'D' )
	{
		ucState = Waiting;
		*ucPtrTime = 0;	*( ucPtrTime + 1 ) = 0;	*( ucPtrTime + 1 ) = 0;
		uiTurns = 0;
		uiLayers = 0;
		ucStepY = 0;
		uiStepX = 0;
		LCDConstString2Spot( 1, 11, cBlankLine );				//erase
		LCDConstString2Spot( 2, 11, cBlankLine );				//		wholescreen
		LCDConstString2Spot( 2, 1, cWaitingMess );
	}
}


/*************************************************************************************************************************************************
	PauseWinding() checks to see if wish to resume winding.
*************************************************************************************************************************************************/
void PauseWinding( char cKey )
{
	if( cKey == 'D' )
	{
		ucState = Winding;										//start both step motors
		PIE1 |= 0x04;											//enable CCP1 ints, set CCP1IE
		PIE2 |= 0x01;											//enable CCP2 ints, set CCP12E
		CCP1CON = 0x08;											//Compare mode, set output on match
		CCP2CON = 0x08;											//Compare mode, set output on match
		INTCON |= 0x40;											//enable peripheral ints
		LCDConstString2Spot( 1, 11, cCoilStepsBlank );			//clear Coil steps
		LCDConstString2Spot( 2, 11, cLevelerStepsBlank );		//				and Leveler steps when done jogging
		LCDConstString2Spot( 2, 1, cWindingMess );
	}
}


/*************************************************************************************************************************************************
	PauseJog2() checks to see if wish to jog.
*************************************************************************************************************************************************/
void PauseJog2( char cKey )
{
	if( (cKey == '*') || (cKey == '#') )
	{
		ucState = Jogging2;
		LCDConstString2Spot( 2, 1, cJogging2Mess );
	}
}


/*************************************************************************************************************************************************
	Jog2Pause() checks to see if done jogging.
*************************************************************************************************************************************************/
void Jog2Pause( char cKey )
{
	if( cKey == 'D' )
	{
		ucState = Pause;
		LCDConstString2Spot( 2, 1, cPauseMess );
	}
}


/*************************************************************************************************************************************************
Jog2Y() jogs the Y axis (winder) and shows Coil winder position: RB2 is direction control and step pulses are produced by (CCP1)RC2
*************************************************************************************************************************************************/
void Jog2Y( char * cPtrTurnsString, char * cPtrStepYString )
{
//If '9' button is pressed,
	PORTA |= 0x04;												//Energize ROW3
	while( COL3 )												//do this as long as '#' is held down
	{
		PORTC &= ~0x01;											//manipulate direction pin(RC0) for CCW
		if( PORTC & 0x04 )										//Was the pin high? If Yes
		{
			PORTC &= ~0x04;										//						drive it low
		}
		else													//					If No,
		{
			PORTC |= 0x04;										//						drive it high
			ucStepY--;											//						If Yes, uncount a Y step.
			if( ucStepY == 0 )									//Have we decremented to 0?
			{
				ucStepY = 200;									//If Yes, reset the coil step count
				uiTurns--;										//							and uncount a Turn
				LCDConstString2Spot( 1, 7, cBlankTurns );
				xtoa( uiTurns, uiWeightTable, cPtrTurnsString );
				LCDString2Spot( 1, 7, cPtrTurnsString );
			}
		}
		__delay_ms( 5 );
		xtoa( ucStepY, uiWeightTable, cPtrStepYString );
		LCDConstString2Spot( 1, 12, cBlankStepY );				//need to blank every pass for decrementing steps
		LCDString2Spot( 1, 12, cPtrStepYString );
	}
	PORTA &= ~0x04;												//De-energize ROW3

//If '#' button is pressed,
	PORTA |= 0x08;												//Energize ROW4
	while( COL3 )												//do this as long as '9' is held down
	{
		PORTC |= 0x01;											//manipulate direction pin(RC0) for CW
		if( PORTC & 0x04 )										//Was the pin high? If Yes
		{
			PORTC &= ~0x04;										//						drive it low
		}
		else													//					If No,
		{
			PORTC |= 0x04;										//						drive it high
			ucStepY++;											//						If Yes, count a Y step.
			if( ucStepY == 200 )								//Have we incremented past 200?
			{
				ucStepY = 0;									//If Yes, reset the coil step count
				uiTurns++;										//							and count a Turn
				LCDConstString2Spot( 1, 12, cBlankStepY );
				LCDConstString2Spot( 1, 7, cBlankTurns );
				xtoa( uiTurns, uiWeightTable, cPtrTurnsString );
				LCDString2Spot( 1, 7, cPtrTurnsString );
			}
		}
		__delay_ms( 5 );
		xtoa( ucStepY, uiWeightTable, cPtrStepYString );
		LCDString2Spot( 1, 12, cPtrStepYString );
	}
	PORTA &= ~0x08;												//De-energize ROW4

	PORTC |= 0x01;												//y axis always turns CW

}


/*************************************************************************************************************************************************
Jog2X() jogs the X axis (leveler) and shows Leveler position: RB1 is direction control and step pulses are produced by (CCP2)RC1
*************************************************************************************************************************************************/
void Jog2X( char * cPtrLayersString, char * cPtrStepXString  )
{
	unsigned char ucCopyC;
	ucCopyC = PORTC & 0x08;										//keep a copy of the state of RC3

//If '*' button is pressed, want to jog CW
	PORTA |= 0x08;												//Energize ROW4
	while( COL1 )												//do this as long as '*' is held down
	{
		PORTC |= 0x08;											//manipulate direction pin(RC3) for CW
		if( PORTC & 0x02 )										//Was the pin high? If Yes
		{
			PORTC &= ~0x02;										//						drive it low
		}
		else													//					If No,
		{
			PORTC |= 0x02;										//						drive it high
			uiStepX--;											//						If Yes, uncount a X step.
			if( uiStepX == 0 )									//Have we decremented to 0?
			{
				uiStepX = uiLayerLength;						//If Yes, reset the Layer step count
				uiLayers--;										//								and uncount a Layer
				LCDConstString2Spot( 2, 7, cBlankLayers );
				xtoa( uiLayers, uiWeightTable, cPtrLayersString );
				LCDString2Spot( 2, 7, cPtrLayersString );
			}
		}
		__delay_ms( 5 );
		xtoa( uiStepX, uiWeightTable, cPtrStepXString );
		LCDConstString2Spot( 2, 12, cBlankStepX );				//need to blank every pass for decrementing steps
		LCDString2Spot( 2, 12, cPtrStepXString );
	}
	PORTA &= ~0x08;												//de-energize ROW4

//If '7' button is pressed, want to jog CCW
	PORTA |= 0x04;												//Energize ROW3
	while( COL1 )												//do this as long as '7' is held down
	{
		PORTC &= ~0x08;											//manipulate direction pin for CCW
		if( PORTC & 0x02 )										//Was the pin high? If Yes
		{
			PORTC &= ~0x02;										//						drive it low
		}
		else													//					If No,
		{
			PORTC |= 0x02;										//						drive it high
			uiStepX++;											//						If Yes, count a X step.
			if( uiStepX == uiLayerLength )						//Have we incremented past LayerLength?
			{
				uiStepX = 0;									//If Yes, reset the Layer step count
				uiLayers++;										//count a layer
				LCDConstString2Spot( 2, 12, cBlankStepX );
				LCDConstString2Spot( 2, 7, cBlankLayers );
				xtoa( uiLayers, uiWeightTable, cPtrLayersString );
				LCDString2Spot( 2, 7, cPtrLayersString );
			}
		}
		__delay_ms( 5 );
		xtoa( uiStepX, uiWeightTable, cPtrStepXString );
		LCDString2Spot( 2, 12, cPtrStepXString );
	}
	PORTA &= ~0x04;												//De-energize ROW3

	if( ucCopyC ) PORTC |= 0x08;								//put the X-axis direction pin back the way it was
	else PORTC &= ~0x08;
}


/*************************************************************************************************************************************************
	DoneJog1()
*************************************************************************************************************************************************/
void DoneJog1( char cKey )
{
	if( cKey == 'D' )											//cKey will be true if 'D' was pressed and released
	{
		LCDConstString2Spot( 2, 1, cWaitingMess );
		ucState = Waiting;
	}
}


/*************************************************************************************************************************************************
	Jog1Y() jogs the Y axis
*************************************************************************************************************************************************/
void Jog1Y( void )
{
//If '#' button is pressed,
	PORTA |= 0x08;												//Energize ROW4
//	PORTA |= ROW4;												//Energize ROW4
	while( COL3 )												//do this as long as '#' is held down
	{
		PORTC |= 0x01;											//manipulate direction pin(RC0) for CW
		if( PORTC & 0x04 )										//Was the pin high? If Yes
		{
			PORTC &= ~0x04;										//						drive it low
		}
		else													//					If No,
		{
			PORTC |= 0x04;										//						drive it high
		}
		__delay_ms( 5 );
	}
	PORTA &= ~0x08;												//de-energize ROW4
//	PORTA &= ~ROW4;												//De-energize ROW4

//If '9' button is pressed,
	PORTA |= 0x04;												//Energize ROW3
//	PORTA |= ROW3;												//Energize ROW3
	while( COL3 )												//do this as long as '9' is held down
	{
		PORTC &= ~0x01;											//manipulate direction pin(RC0) for CCW
		if( PORTC & 0x04 )										//Was the pin high? If Yes
		{
			PORTC &= ~0x04;										//						drive it low
		}
		else													//					If No,
		{
			PORTC |= 0x04;										//						drive it high
		}
		__delay_ms( 5 );
	}
	PORTA &= ~0x04;												//De-energize ROW3
//	PORTA &= ~ROW3;												//De-energize ROW3

	PORTC |= 0x01;												//y axis always turns CW	
}


/*************************************************************************************************************************************************
	Jog1X() jogs the X axis
*************************************************************************************************************************************************/
void Jog1X( void )
{
	unsigned char ucCopyC;
	ucCopyC = PORTC & 0x08;										//keep a copy of the state of RC3

//If '*' button is pressed, want to jog CW
	PORTA |= 0x08;												//Energize ROW4
//	PORTA |= ROW4;												//Energize ROW4
	while( COL1 )												//do this as long as '*' is held down
	{
		PORTC |= 0x08;											//manipulate direction pin(RC3) for CW
		if( PORTC & 0x02 )										//Was the pin high? If Yes
		{
			PORTC &= ~0x02;										//						drive it low
		}
		else													//					If No,
		{
			PORTC |= 0x02;										//						drive it high
		}
		__delay_ms( 1 );
	}
	PORTA &= ~0x08;												//de-energize ROW4
//	PORTA &= ~ROW4;												//De-energize ROW4

//If '7' button is pressed, want to jog CCW
	PORTA |= 0x04;												//Energize ROW3
//	PORTA |= ROW3;												//Energize ROW3
	while( COL1 )												//do this as long as '7' is held down
	{
		PORTC &= ~0x08;											//manipulate direction pin(RC3) for CCW
		if( PORTC & 0x02 )										//Was the pin high? If Yes
		{
			PORTC &= ~0x02;										//						drive it low
		}
		else													//					If No,
		{
			PORTC |= 0x02;										//						drive it high
		}
		__delay_ms( 1 );
	}
	PORTA &= ~0x04;												//De-energize ROW3
//	PORTA &= ~ROW3;												//De-energize ROW3

	if( ucCopyC ) PORTC |= 0x08;								//put the X-axis direction pin back the way it was
	else PORTC &= ~0x08;
}


/*************************************************************************************************************************************************
	Finished() checks to see if we are done winding. A state change happens when done.
*************************************************************************************************************************************************/
void Finished( void )
{
	if( uiTurns >= uiTurnsMax )									//Are we done winding? If Yes, stop both motors.
	{
		PIE1 &= ~0x04;											//disable CCP1 ints, clear CCP1IE
		PIE2 &= ~0x01;											//disable CCP2 ints, clear CCP12E
		CCP1CON = 0;											//Compare off
		CCP2CON = 0;											//Compare off
		INTCON &= ~0x40;										//disable peripheral ints
		ucState = Done;
	}
}


/*************************************************************************************************************************************************
	WindingPause() may cause a state change depending on button presses. No button press will be no state change.
*************************************************************************************************************************************************/
void WindingPause( char cKey, char * cPtrStepXString, char * cPtrStepYString )
{
	if( cKey != ' ' )											//any button press will cause a state change
	{
		ucState = Pause;
		LCDConstString2Spot( 2, 1, cPauseMess );
		CursorXY( 11, 1 ); LCDPutc( 'S' );						//'S' for Winder steps
		CursorXY( 11, 2 ); LCDPutc( 'S' );						//'S' for Leveler steps
		xtoa( uiStepX, uiWeightTable, cPtrStepXString );
		LCDString2Spot( 2, 12, cPtrStepXString );				//show Leveler steps
		xtoa( ucStepY, uiWeightTable, cPtrStepYString );
		LCDString2Spot( 1, 12, cPtrStepYString );				//show Coil steps
		PIE1 &= ~0x04;											//disable CCP1 ints, clear CCP1IE
		PIE2 &= ~0x01;											//disable CCP2 ints, clear CCP12E
		CCP1CON = 0x00;											//Compare off
		CCP2CON = 0x00;											//Compare off
		INTCON &= ~0x40;										//disable peripheral ints
	}
}


/*************************************************************************************************************************************************
	GoJog1() may cause a state change depending on button presses. No button press will be no state change.
*************************************************************************************************************************************************/
void GoJog1( unsigned char cKey )
{
	if( (cKey == '#') || (cKey == '*') )
	{
		ucState = Jogging1;
		LCDConstString2Spot( 2, 1, cJogging1Mess );
	}
	if( cKey == 'D' )											//start both step motors
	{
		PIE1 |= 0x04;											//enable CCP1 ints, set CCP1IE
		PIE2 |= 0x01;											//enable CCP2 ints, set CCP12E
		CCP1CON = 0x08;											//Compare on
		CCP2CON = 0x08;											//Compare on
		INTCON |= 0x40;											//enable peripheral ints
		ucState = Winding;
		LCDConstString2Spot( 2, 1, cWindingMess );
		CursorXY( 6, 1 ); LCDPutc( 'T' ); LCDPutc( '0' );
		CursorXY( 6, 2 ); LCDPutc( 'L' ); LCDPutc( '0' );
	}
}





static void interrupt
ISR( void )
{

/*************************************************************************************************************************************************
		Timer 0 ISR: This interrupt happens every 16.384ms.
*************************************************************************************************************************************************/
	if( INTCON & 0x04 )								//Is TMR0IF set?
	{
		ucTicCount ++;
		ucTicFlag = 1;								//show that a tick has ocurred
		if( ucTicCount >= 62 )						// 16.384ms x 61 0.999 sec
		{
			ucTicCount = 1;
			ucPass++;
		}
		INTCON &= ~0x04;							//clear TMR0IF
		//PORTB ^= 0x08;							//toggle led
	}

/*************************************************************************************************************************************************
		CCP1 ISR: Produces steps for the Spool designated as Y axis, step signal is on RC2
*************************************************************************************************************************************************/
	if( PIR1 & 0x04 )								//Is CCP1IF set?
	{
		if( !(CCP1CON &  0x01) )					//if CCP1CON bit 0 was 0
			ucStepY++;								//						count a Y step
		if( ucStepY >= 200 )						//200 steps per turn
		{
			ucStepY = 0;
			uiTurns++;
		}
		uiComp1 += uiCp1;							//preload CCPR1
		CCPR1H = uiComp1 >> 8;
		CCPR1L = uiComp1;
		CCP1CON ^= 0x01;							//Change the compare action on next int.
		PIR1 &= ~0x04;								//clear CCP1IF
	}

/*************************************************************************************************************************************************
		CCP2 ISR: Produces steps for the Leveler designated as X axis
*************************************************************************************************************************************************/
	if( PIR2 & 0x01 )								//Is CCP2IF set?
	{
		if( !(CCP2CON & 0x01) )						//if CCP1CON bit 0 was 0
			uiStepX++;								//						count a X step
		if( uiStepX >= uiLayerLength )				//LayerLength steps per layer
		{
			uiStepX = 0;
			uiLayers++;
			if( uiLayers & 0x0001 )					//RC3 is lo for even layers and hi for odd layers
				PORTC |= 0x08;
			else PORTC &= ~0x08;
		}
		uiComp2 += uiCp2;							//preload CCPR2
		CCPR2H = uiComp2 >> 8;
		CCPR2L = uiComp2;
		CCP2CON ^= 0x01;							//Change the compare action on next int.
		PIR2 &= ~0x01;								//clear CCP2IF
	}
}
